#!/bin/bash

zpaq_setup() {

	local binpath=${ROOTDIR}/plugins/bin/zstd/zstd.exe

	# Install lz4 if it doesn't already exist

	if [[ ! -f ${binpath} ]] ; then
		(
			mkdir -p ${ROOTDIR}/plugins/bin || true
			cd ${ROOTDIR}/plugins/bin
			
			git clone https://github.com/facebook/zstd
			cd zstd
			export CC=x86_64-w64-mingw32-gcc
			make
		)
	fi

	local version=$( ${binpath} --help | head -n1 | grep -Eo 'v[0-9][^, ]+' )
	
	# Register compression targets

	for level in {1..19} ; do
		compressor_register \
			"zstd ${version}" \
			"zstd -${level}" \
			"${binpath} -${level} > %OUTFILE% < %INFILE%"
	done
	for level in {20..22} ; do
		compressor_register \
			"zstd ${version}" \
			"zstd -${level}" \
			"${binpath} -${level} --ultra > %OUTFILE% < %INFILE%"
	done

}

zpaq_setup
