#!/bin/bash

set -eu
ROOTDIR=$(pwd)
COMPRESSORS=()
export TIMEFORMAT=%R

compressor_measure() {
	local data="$(cat)"
	local input="$1"
	
	local inputsize=$(stat -c %s "${input}")
	local family=$(echo "$data" | cut -d':' -f1)
	local detail=$(echo "$data" | cut -d':' -f2)
	
	local target=$(mktemp)
	if [[ -f ${target} ]] ; then
		rm "${target}"
	fi
	
	local cmd=$(
		echo "$data" | cut -d':' -f3- \
			| sed -re "s~%INFILE%~${input}~" \
			| sed -re "s~%OUTFILE%~${target}~"
	)

	#echo "$cmd"
	
	local duration="$( ( time eval "$cmd" < /dev/null ) 2>&1 | tail -n1 )"
	local size="$(stat -c %s "${target}")"
	
	echo "${family}"$'\t'"${detail}"$'\t'${duration}$'\t'${inputsize}$'\t'${size}
	
	rm "${target}"
}

compressor_register() {
	local family="$1"
	local detail="$2"
	local command="$3"
	
	COMPRESSORS+=( "$(echo -n "${family}:${detail}:${command}")" )
}

main() {
	
	# Run all plugins
	echo "Loading plugins, please wait..."
	for pluginfile in plugins/*.sh ; do
		. $pluginfile
	done
	
	# Run a full test of all corpus files
	local curdate="$(date +%s)"
	
	local totaltasks=$(( $(ls test-corpus | wc -l) * ${#COMPRESSORS[*]} ))
	local donect=0
	local donepct=0
	
	for testfile in test-corpus/* ; do
		local resultfile="results-${curdate}-$(basename "${testfile}").log"
		local shortname=$( basename "$testfile" )
		touch "$resultfile"
	
		echo $'#family\tflags\tduration\tinputsize\toutputsize' >> $resultfile
	
		for cfg in "${COMPRESSORS[@]}" ; do
		
			local donepct=$(( ${donect} * 100 / ${totaltasks} ))		
			local family=$(echo "$cfg" | cut -d':' -f1)
			local detail=$(echo "$cfg" | cut -d':' -f2)
			
			echo "$donepct" | dialog --gauge "      File: ${shortname}"$'\n'"Compressor: $family ($detail)" 8 70 0
			
			echo "${cfg}" | compressor_measure "$testfile" >> $resultfile
			
			donect=$(( ${donect} + 1 ))
		done
	done
	
	echo 100 | dialog --gauge "All tasks completed." 8 70 0
}

main "$@"
